(** Template file for Pactole *)


(* Helping typeclass resolution avoid infinite loops. *)
Typeclasses eauto := (bfs).

(* The general Pactole framework. *)
Require Import Pactole.Setting.

(* The space in which robots evolve.  (in subdirectory Spaces/)
   Possible choices include: R, R2, Ring, ... *)
Require Import Rbase.
Require Import Pactole.Spaces.R2.
Instance Space : Location := {| location := R2 |}.
(* Here Coq finds the additional required fields inside Pactole.Spaces.R2, as we can see with *)
Print Space.

(* The type of spectrum.  (in subdirectory Spectra/)
   Possible choices include: multiset, set, their variants with limited radius, ... *)
Require Import Pactole.Spectra.SetSpectrum.

(* Optionally, you can add constraint on the model such as rigid move, or using simlarities. *)
Require Import Pactole.Models.Similarity.
Require Import Pactole.Models.Rigid.


(* Define the number of robots: first the number of good ones, then the number of byzantine ones. *)
Parameter n : nat.
Axiom n_non_0 : n <> 0%nat.
Instance MyRobots : Names := Robots (2 * n) 0.
(* Here, we have [2 * n] good robots and no byzantine ones, with [n] arbitrary but non null. *)


(* Define what the space and the state of a robot are.
   Here the state only contains the position of the robot, which is in the real plane.
   The first [R2] is the space, whereas the second one is the state (which must contain the space). *)
Instance Info : State location := OnlyLocation.

(* What can demons choose to change of frame of reference?
   Here we use similarities. *)
Instance FC : frame_choice (Similarity.similarity R2) := FrameChoiceSimilarity.

(* What choices can demons make for the state update?
   Here they do not make any choice, hence we use [unit]. *)
Instance UC : update_choice unit := {update_choice_EqDec := unit_eqdec}.
Instance IC : inactive_choice unit := {inactive_choice_EqDec := unit_eqdec}.

(* Finally, define what the update funciton is. *)
Instance UpdateFun : update_functions Datatypes.unit Datatypes.unit := {
  update := fun _ _ target _ => target ratio_1;
  inactive := fun config id _ => config id }.
Proof.
+ now repeat intro.
+ intros ? ? Heq ? ? ? ? ? ?. subst. apply Heq.
Defined.

(* If you have constraints on your model, now is the time to prove them.
   Here, we prove that our update function is indeed rigid. *)
Instance Update : RigidUpdate.
Proof. split. now intros. Qed.

(* The setup is now complete, you can start working. *)

(** **  Exercise 1: Formalization of problems  **)

(** In the following, whenever relevant, we assume that we are in a FSYNC setting.
    Remember that solving a problem is ultimately a property of a [robogram], not of an [execution]. *)

(** ***  The termination problem **)

(** All (good) robots should stop moving in finite time. *)









(** ***  The gathering problem  **)

(** All good robots must reach the same location, unknown beforehand, in finite time and stay there. *)

(* Hint: Define it in steps: - Be at a given point;
                             - Stay at a given point;
                             - Reach a given point in finite time and then stay there;
                             - Solve the Gathering.

   You can have a look at the stream operators inside Util/Streams. *)
Print Stream.instant.
Print Stream.forever.
Print Stream.eventually.








(** ***  The convergence problem  **)

(** In finite time, all robots must be within a disk of radius epsilon and stay there.
    Epsilon is unknown to the robots, that is, the robogram must work for any epsilon > 0. *)
Open Scope R_scope. (* get access to the notations of R (in particular <=) *)









(** ***  Exploration with stop  **)

(** Explore all locations in your space (assumed finite!) then stop. *)









(** **  Exercise 2: Properties of demons  **)

(** ***  Fully synchornous demons  **)

(** Define what it means for a demon to be fully synchronous.

    You can use [activate : demonic_action -> ident -> bool]. *)







(** ***  Fair demon  **)







(** **  Exercise 3: Robots with lights  **)

(** In order to communicate with others, a robot can have lights, with different colors (possible values).
    There are three types of lights:
    - internal lights are only seen by the robot itself (they act as a finite memory in that case);
    - external lights are only visible by others robots (a robot does not see its light);
    - full lights are visible by everyone.

    What part of the framework should be changed to accomodate lights?
    Define the new instance to get a new framework for robots with lights.

    You can use  *)

Section RobotsWithLights.






End RobotsWithLights.

(** **  Exercise 4: Temporal logic  *)

(** Define the "until" and "weak until" stream operators.
    A stream satisfies [P until Q] if it satisfies [P] up to some point at which it satisfies [Q].
    A stream satisfies [P weak until Q] if it satisfies [forever P] or it satisfies [P until Q].
 *)







(** ** Exercise 5: Comparison between problems  **)

(** State and prove that any robogram solving the gathering problem also solves the convergence one. *)








(** **  Exercise 6: Formalization and proof for the stop problem (never moving)  **)

(** Here we consider a dummy example where robots never move, and thus they solve the stop problem.
    We do not even have to make any assumption on the demon. *)









(** **  Exercise 7: Formalization and proof of gathering in the FSYNC setting  **)

(** Reusing your specification for the gathering problem, prove that the robogram trying to go
    towards the barycenter indeed solves the gathering problem in the FSYNC setting.

    Using Exercise 5, deduce that it also solve the convergence problem in the same setting. *)




